/* vim: set tabstop=8 softtabstop=2 shiftwidth=2 expandtab: */
/*
 *    Copyright (C) 2011 Fabian Deutsch
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

using Presence;

class AppPresence : GLib.Object
{
    static string? uri;
    static bool is_server = false;
    static bool is_publisher = false;

    static bool is_not_decorated = false;
    static bool is_ipv6 = false;
    static double video_scale_factor = 0.5;
    static double video_quality_factor = 0.7;
    static double video_fps_factor = 0.5;
    static bool is_debug = false;
    static bool use_advanced_video_preferences = false;

    const OptionEntry[] options = {
        { "uri", 'u', 0, OptionArg.STRING, ref uri, "The RTSP URI to connect to.", "rtsp://127.0.0.1:8554/test" },
        { "server", 's', 0, OptionArg.NONE, ref is_server, "Provide an RSTP stream for the first device.", null },
        { "publish", 'p', 0, OptionArg.NONE, ref is_publisher, "Publish any RTSP stream.", null },
        { "without-window-decorations", 0, 0, OptionArg.NONE, ref is_not_decorated, "Hide window decorations.", null },
        { "ipv6", '6', 0, OptionArg.NONE, ref is_ipv6, "Use IPv6.", null },
        { "video-scale-factor", 0, 0, OptionArg.DOUBLE, ref video_scale_factor, "Factor used to scale the video (0-1).", null },
        { "video-quality-factor", 0, 0, OptionArg.DOUBLE, ref video_quality_factor, "Factor used to set the compression quality of video (0-1).", null },
        { "video-fps-factor", 0, 0, OptionArg.DOUBLE, ref video_fps_factor, "Factor used to set the FPS of the video (0-1).", null },
        { "debug", 'd', 0, OptionArg.NONE, ref is_debug, "Output some debugging informations.", null },
        { "advanced-video-preferences", 0, 0, OptionArg.NONE, ref use_advanced_video_preferences, "If more advanced video preferences should be presented.", null },
        { null }
    };


    ApplicationSettings settings;

    MainClientWindow main_window;
    Notify.Notification notice;

    LibPresence provider;

    public AppPresence ()
    {
        this.settings = new ApplicationSettings ();
        this.provider = new LibPresence.with_settings (this.settings);
    }


    public void init (ref unowned string[] args)
    {
        parse_options (ref args);

        this.provider.init (ref args);

        init_ui (args);

        bind_lib_to_ui ();

        apply_settings ();
    }

    void parse_options (ref unowned string[] args)
    {
        try {
            var opt_context = new OptionContext ("- Show presence");
            opt_context.set_help_enabled (true);
            opt_context.add_main_entries (options, null);
            opt_context.parse (ref args);
        } catch (OptionError e) {
            stdout.printf ("%s\n", e.message);
            stdout.printf ("Run '%s --help' to see a full list of available command line options.\n", args[0]);
            return;
        }

        /* Assign this early to track property changes */
        this.settings.debug = is_debug;
    }

    void init_ui (string[] args)
    {
        Notify.init ("presence");

        this.main_window = new MainClientWindow (args, this.settings);
        this.main_window.decorated = !is_not_decorated;
    }

    void bind_lib_to_ui ()
    {
        this.provider.peers_discovered.connect ( (s) => {
            this.notice = new Notify.Notification ("New peers were discovered.", 
                "Presence discovered new peers. Use the peers dialog to connect to one of them.", "face-glasses");
            this.notice.show ();
            this.main_window.set_peers (this.provider.peers);
        });
        this.provider.peers_lost.connect ( (s) => {
#if DEBUG
            debug ("Updating peers (rm).");
#endif
            this.main_window.set_peers (this.provider.peers);
        });

        this.provider.bind_property ("rtsp-address", this.main_window, 
                                     "rtsp-address", BindingFlags.SYNC_CREATE);
    }

    void apply_settings ()
    {
        this.settings.rtsp_server_enabled = is_server;
        this.settings.ipv6 = is_ipv6;
        if (uri != null)
        {
            this.settings.rtsp_client_uri = uri;
            this.settings.rtsp_client_enabled = true;
        }

        this.settings.mdns_name = "%s@%s".printf (
            Environment.get_user_name (), 
            Environment.get_host_name ()
        );
        this.settings.mdns_enabled = is_publisher;

        this.settings.video_scale_factor = video_scale_factor;
        this.settings.video_quality_factor = video_quality_factor;
        this.settings.video_fps_factor = video_fps_factor;

        this.settings.use_advanced_video_preferences_dialog = 
                                                use_advanced_video_preferences;
    }

    public void run ()
    {
        this.main_window.show ();
        this.main_window.run ();
    }
}

