/* vim: set tabstop=8 softtabstop=2 shiftwidth=2 expandtab: */
/*
 *    Copyright (C) 2011 Fabian Deutsch
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

class ChildDialog : Mx.Dialog
{
    protected Clutter.Stage parent;
    protected ApplicationSettings application_settings;

    protected Clutter.Script script;

    public ChildDialog (Clutter.Stage w, ApplicationSettings s, string? filename = null)
    {
        this.parent = w;
        this.application_settings = s;
        
        this.set_transient_parent (this.parent);
        
        build_dialog (filename);
    }
    
    void build_dialog (string? filename)
    {
        script = new Clutter.Script ();

        if (filename != null)
        {
            load_content (filename);
        }

        var a = new Mx.Action.full ("close", "Close", () => {
            this.hide ();
        });
        this.add_action (a);
    }

    void load_content (string filename)
    {
        try {
            script.load_from_file (GLib.Path.build_path (
                GLib.Path.DIR_SEPARATOR_S, BuildConfig.PACKAGE_DATADIR, 
                BuildConfig.PACKAGE_NAME, filename));
        } catch (Error e) {
            error ("Failed to load dialog from '%s': %s", filename, e.message);
        }

        this.add_actor (script.get_object ("content") as Clutter.Actor);
    }
}


class BridgeDialog : ChildDialog
{
    protected struct WidgetPropertyBridge
    {
        unowned string widget_name;
        unowned string widget_property;
        unowned string settings_property;
        public string to_string () { return @"$(widget_name).$(widget_property) with $(settings_property)"; }
    }

    public BridgeDialog (Clutter.Stage w, ApplicationSettings s, string filename, WidgetPropertyBridge[] bridges)
    {
        base (w, s, filename);

        apply_bridges (bridges);
    }

    void apply_bridges (WidgetPropertyBridge[] bridges)
    {
        foreach (unowned WidgetPropertyBridge bridge in bridges)
        {
#if DEBUG
            debug (@"Binding $(bridge) via bridge.");
#endif
            var ts = script.get_object (bridge.widget_name);
            ts.bind_property (bridge.widget_property, this.application_settings, 
                bridge.settings_property, BindingFlags.BIDIRECTIONAL);
        }
    }
}

class SettingsDialog : BridgeDialog
{
    public SettingsDialog (Clutter.Stage w, ApplicationSettings s)
    {
        const BridgeDialog.WidgetPropertyBridge[] bridges = {
            {"toggle_mute", "active", "is-muted"},
            {"toggle_serve", "active", "rtsp-server-enabled"},
            {"toggle_publish", "active", "mdns-enabled"},
            {"toggle_receive", "active", "rtsp-client-enabled"},
            {"entry_uri", "text", "rtsp-client-uri"},
            {"entry_name", "text", "mdns-name"},
            {"entry_password", "text", "rtsp-server-auth-password"}
        };

        base (w, s, "/settingsDialog.json", bridges);
    }
}

string[] available_video_devices()
{
  string[] devices = new string[0];
  GUdev.Client client = new GUdev.Client (null);
  List<GUdev.Device> devs = client.query_by_subsystem("video4linux");
  foreach (GUdev.Device d in devs)
  {
      devices += d.get_device_file ();
  }
  return devices;
}

class VideoSettingsDialog : BridgeDialog
{
    SelectionExpander e;

    public VideoSettingsDialog (Clutter.Stage w, ApplicationSettings s)
    {
        const BridgeDialog.WidgetPropertyBridge[] bridges = {
            {"video_size_factor", "value", "video-scale-factor"},
            {"video_quality_factor", "value", "video-quality-factor"},
            {"video_fps_factor", "value", "video-fps-factor"}
        };

        base (w, s, "/videoSettingsDialog.json", bridges);

        add_expander ();
    }

    void add_expander ()
    {
      SelectionExpander.Option[] entries = new SelectionExpander.Option[0];

      foreach (string s in available_video_devices())
      {
/*          entries += SelectionExpander.Option () { 
                        name = d.get_device_file (),
                        label = d.get_name ()
                     };*/
          entries += SelectionExpander.Option () {
                        name = s,
                        label = s
                      };
      }
      e = new SelectionExpander  (entries);
      e.selected.connect ( (name) => {
        this.application_settings.video_device = name;
      });

      var t = script.get_object ("VideoSettingsTable") as Mx.Table;
      t.add_actor (e);
      var c = t as Clutter.Container;
      c.child_set_property (e, "row", 0);
      c.child_set_property (e, "column", 1);
    }
}


class PeersDialog : ChildDialog
{
    Mx.Table peers_table;

    public signal void secondary_changed (string uri, bool enabled);

    public PeersDialog (Clutter.Stage w, ApplicationSettings s)
    {
        base (w, s, "peersDialog.json");

        build_content ();
    }
    
    void build_content ()
    {
        this.peers_table = script.get_object ("PeersTable") as Mx.Table;
    }

    public void set_peers (HashTable<string,string> peers)
    {
        this.peers_table.foreach ( (a) => { a.destroy (); } );

        int idx = 0;
        peers.foreach ( (n,u) => {
            add_peer_entry (n, u, idx++);
        });

        if (idx == 0)
        {
            this.peers_table.add_child (new Mx.Label.with_text ("No peers found."), 0, 0);
        }
    }

    void add_peer_entry (string name, string uri, int idx)
    {
        var l = new Mx.Label.with_text ("%s (%s)".printf (name, uri));
        var b = new Mx.Button.with_label ("Connect");
        //b.is_toggle = true;
        
        /* Change primary client if changed */
        b.clicked.connect ( () => {
            //if (b.toggled)
            {
              this.application_settings.rtsp_client_enabled = false;
              this.application_settings.rtsp_client_uri = uri;
              this.application_settings.rtsp_client_enabled = true;
              this.hide ();
            }
            /*else
            {
              this.application_settings.rtsp_client_enabled = false;
            }*/
        });
        
        /* Update toggle state if primary client is changed */
        /*this.application_settings.notify["rtsp-client-enabled"].connect ( () => {
          b.toggled = (uri == this.application_settings.rtsp_client_uri);
        });*/

        var s = new Mx.Button.with_label ("Secondary");
        s.clicked.connect ( () => {
            //this.secondary_changed (uri, s.toggled);
            var t = this.application_settings.rtsp_secondary_client_uris;
            t += uri;
            this.application_settings.rtsp_secondary_client_uris = t;
        });

        this.peers_table.add_child (l, idx, 0);
        this.peers_table.add_child (b, idx, 1);
        this.peers_table.add_child (s, idx, 2);

        this.peers_table.child_set (l, "y-fill", false);
        this.peers_table.child_set (l, "y-align", Mx.Align.MIDDLE);
        this.peers_table.child_set (b, "y-align", Mx.Align.MIDDLE);
    }
}


class OneClickStreamingDialog : BridgeDialog
{
    Mx.ComboBox video_device;
    Mx.ComboBox video_quality;
    Mx.ComboBox video_fps;
    Mx.ComboBox video_size;

    Mx.Action start_action;

    string[] qualities = {"bad", "good", "best"};
    float[] quality_factors = { 0.1f, 0.7f, 1.0f };
    string[] sizes = {"320x180", "640x360", "1280x720"};
    float[] size_factors = { 0.25f, 0.5f, 1.0f };
    string[] fps = {"10 fps", "20 fps", "30 fps"};
    float[] fps_rates = { 10, 20, 30 };

    public OneClickStreamingDialog (Clutter.Stage w, ApplicationSettings s)
    {
        const BridgeDialog.WidgetPropertyBridge[] bridges = {/*
            {"video_size_factor", "value", "video-scale-factor"},
            {"video_quality_factor", "value", "video-quality-factor"},
            {"video_fps_factor", "value", "video-fps-factor"}*/
        };

        base (w, s, "/oneclickDialog.json", bridges);

        this.video_device = script.get_object ("video_device") as Mx.ComboBox;
        this.video_quality = script.get_object ("video_quality") as Mx.ComboBox;
        this.video_fps = script.get_object ("video_fps") as Mx.ComboBox;
        this.video_size = script.get_object ("video_size") as Mx.ComboBox;

        add_entries();
        add_actions();
    }

    void add_entries()
    {
      this.video_device.notify["index"].connect( () => {
        this.application_settings.video_device = this.video_device.active_text;
      });

      this.video_size.notify["index"].connect( () => {
        float f = size_factors[this.video_size.index];
        this.application_settings.video_scale_factor = f;
      });

      this.video_quality.notify["index"].connect( () => {
        float f = quality_factors[this.video_quality.index];
        this.application_settings.video_quality_factor = f;
      });

      this.video_fps.notify["index"].connect( () => {
        float f = fps_rates[this.video_fps.index];
        this.application_settings.video_fps_request = f;
      });

      foreach (var e in available_video_devices())
        this.video_device.append_text(e);
      this.video_device.index = 0;

      foreach (var e in qualities)
        this.video_quality.append_text(e);
      this.video_quality.index = 1;

      foreach (var e in sizes)
        this.video_size.append_text(e);
      this.video_size.index = 1;

      foreach (var e in fps)
        this.video_fps.append_text(e);
      this.video_fps.index = 0;

      this.video_fps.notify_property("index");
    }

    void add_actions()
    {
      start_action = new Mx.Action.full("Start streaming", 
                            "Start streaming", 
                            () => {
                              this.application_settings.rtsp_server_enabled = false;
                              this.application_settings.mdns_enabled = false;

                              this.application_settings.rtsp_server_enabled = true;
                              this.application_settings.mdns_enabled = true;

                              this.hide();
                            });
      this.add_action(start_action);
    }
}
