/* vim: set tabstop=8 softtabstop=2 shiftwidth=2 expandtab: */
/*
 *    Copyright (C) 2011 Fabian Deutsch
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License along
 *    with this program; if not, write to the Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

extern Avahi.EntryGroupService ga_entry_group_add_service(Avahi.EntryGroup group, 
                                                          string name, 
                                                          string type, 
                                                          uint16 port, 
                                                          ...) throws Avahi.Error;

namespace Mdns
{
  /**
   * Handles MDNS based publishing and discovery.
   */
  public class MdnsProvider : GLib.Object
  {
    Avahi.Client client;

    Avahi.EntryGroup entry_group;
    Avahi.EntryGroupService published_service;
    Avahi.ServiceBrowser service_browser;

    public HashTable<string,string> available_services;

    public uint16 port {
      get;
      set;
      default = 8554;
    }

    public string service_type {
      get;
      set;
      default = "_rtsp._tcp";
    }

    public string txt {
      get;
      set;
      default = "presence";
    }

    public bool is_published {
      get;
      set;
      default = false;
    }

    public string name {
      get;
      set;
      default = "no-name-yet";
    }

    public bool is_ipv6 {
      get;
      set;
      default = false;
    }

    public signal void service_discovered (string name, 
                                           string uri, 
                                           string txt);

    public signal void service_lost (string uri);


    Avahi.ServiceResolver service_resolver;

    public MdnsProvider ()
    {
      this.available_services = new HashTable<string,string> (GLib.str_hash, 
                                                              GLib.str_equal);

      this.client = new Avahi.Client();

      try 
      {
        this.client.start();

        init_publish ();
        init_discovery ();
#if DEBUG
        connect_debuging_handlers ();
#endif
      } 
      catch (Error e) 
      {
        error ("An error occured while setting up MDNS: %s", e.message);
      }
    }

    ~MdnsProvider ()
    {
      this.service_resolver = null;
    }
    
    void init_publish () throws Avahi.Error
    {
      this.entry_group = new Avahi.EntryGroup ();
      this.entry_group.attach (this.client);

      this.notify["is-published"].connect (on_is_published_changed_cb);
    }

    void init_discovery () throws Avahi.Error
    {
      this.service_browser = new Avahi.ServiceBrowser (this.service_type);

      this.service_browser.new_service.connect (on_new_service_cb);
      this.service_browser.removed_service.connect (on_removed_service_cb);

      this.service_browser.attach(this.client);
    }

    void connect_debuging_handlers ()
    {
      this.service_discovered.connect ( (name, uri, txt) => {
        debug ("Discovered %s (%s)\n%s", name, uri, txt);
      });
      this.service_lost.connect ( (name) => {
        debug ("Lost %s", name);
      });
    }

    void publish () throws Avahi.Error 
    {
#if DEBUG
      debug ("Publishing '%s' as '%s'", this.name, this.service_type);
#endif
      if (this.published_service != null)
      {
        unpublish ();
      }

      this.published_service = ga_entry_group_add_service (entry_group, 
                                                           this.name, 
                                                           this.service_type, 
                                                           this.port, 
                                                           this.txt);

      this.entry_group.commit ();
    }

    void unpublish () throws Avahi.Error
    {
      this.entry_group.reset ();
    }

    /*
     * Signal handlers
     */
    void on_is_published_changed_cb ()
    {
      try 
        {
          if (is_published)
          {
            publish ();
          }
          else
          {
            unpublish ();
          }
        } 
        catch (Error e) 
        {
          error ("An error occured while toggling publishing: %s", e.message);
        }
    }

    void on_new_service_cb (Avahi.Interface iface, 
                            Avahi.Protocol proto, 
                            string name, 
                            string type, 
                            string domain, 
                            Avahi.LookupResultFlags flags)
    {
#if DEBUG
      debug ("New service found: %s", name);
#endif
      var req_proto = is_ipv6 ? Avahi.Protocol.INET6 : Avahi.Protocol.INET;

      service_resolver = new Avahi.ServiceResolver (iface, proto, name, type, 
                                                    domain, req_proto);

      service_resolver.found.connect (on_service_found_cb);

      try
      {
        service_resolver.attach (this.client);
      }
      catch (Error e)
      {
        error ("Could not attach resolver: %s", e.message);
      }
    }

    void on_removed_service_cb (Avahi.Interface iface, 
                                Avahi.Protocol proto, 
                                string name, 
                                string type, 
                                string domain, 
                                Avahi.LookupResultFlags flags)
    {
      if (this.available_services.remove (name))
      {
#if DEBUG
        debug ("Previously known servicess lost: %s", name);
#endif
        this.service_lost (name);
      }
    }

    void on_service_found_cb (Avahi.Interface iface, 
                              Avahi.Protocol proto, 
                              string name, 
                              string type, 
                              string domain, 
                              string host, 
                              Avahi.Address? addr, 
                              uint16 port, 
                              Avahi.StringList? txt, 
                              Avahi.LookupResultFlags flags)
    {
      // FIXME is there a better way to generate the uri?
      // FIXME why doesnt this work: tpl = is_ipv6 ? "[...]" : "..."; uri = tpl.printf ??
      string uri = "rtsp://%s:%g/%s".printf(addr.to_string (), port, 
                                            (string) txt.text);
      if (is_ipv6)
      {
        uri ="rtsp://[%s]:%g/%s".printf(addr.to_string (), port, 
                                        (string) txt.text); 
      }

      this.available_services[name] = uri;

      this.service_discovered (name, uri, txt.to_string ());
    }
  }
}
